/* GTKstereograph 0.18b, 17/11/2003;
 * Copyright (c) 2000-2003 by Fabian Januszewski <fabian.linux@januszewski.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#define _GNU_SOURCE

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include <time.h>
#include <endian.h>

#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#include <gtk/gtk.h>

// #include "renderer.h"
// #include "gfxio.h"

#include "callbacks.h"
#include "interface.h"
#include "support.h"

extern GtkWidget * GTK_editor (char *filename, void *function);
extern void GTK_editor_set_file( char *filename);

int stereo_gfx = GFX_IO_PNG;

struct GFX_DATA tex_merge;



/* message dialog */

GtkWidget*
create_my_msg_dialog1 (char* title, char* msg)
{
  GtkWidget *msg_dialog1;
  GtkWidget *msg_dialog_vbox3;
  GtkWidget *msg_dialog_label1;
  GtkWidget *dialog_action_area3;
  GtkWidget *msg_dialog_button1;

  msg_dialog1 = gtk_dialog_new ();
  gtk_widget_set_name (msg_dialog1, "msg_dialog1");
  gtk_object_set_data (GTK_OBJECT (msg_dialog1), "msg_dialog1", msg_dialog1);
  gtk_container_set_border_width (GTK_CONTAINER (msg_dialog1), 1);
  gtk_window_set_title (GTK_WINDOW (msg_dialog1), title);
  GTK_WINDOW (msg_dialog1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_position (GTK_WINDOW (msg_dialog1), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (msg_dialog1), FALSE, FALSE, TRUE);

  msg_dialog_vbox3 = GTK_DIALOG (msg_dialog1)->vbox;
  gtk_widget_set_name (msg_dialog_vbox3, "msg_dialog_vbox3");
  gtk_object_set_data (GTK_OBJECT (msg_dialog1), "msg_dialog_vbox3", msg_dialog_vbox3);
  gtk_widget_show (msg_dialog_vbox3);

  msg_dialog_label1 = gtk_label_new (msg);
  gtk_widget_set_name (msg_dialog_label1, "msg_dialog_label1");
  gtk_widget_ref (msg_dialog_label1);
  gtk_object_set_data_full (GTK_OBJECT (msg_dialog1), "msg_dialog_label1", msg_dialog_label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (msg_dialog_label1);
  gtk_box_pack_start (GTK_BOX (msg_dialog_vbox3), msg_dialog_label1, FALSE, FALSE, 2);
  gtk_label_set_line_wrap (GTK_LABEL (msg_dialog_label1), TRUE);

  dialog_action_area3 = GTK_DIALOG (msg_dialog1)->action_area;
  gtk_widget_set_name (dialog_action_area3, "dialog_action_area3");
  gtk_object_set_data (GTK_OBJECT (msg_dialog1), "dialog_action_area3", dialog_action_area3);
  gtk_widget_show (dialog_action_area3);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area3), 10);

  msg_dialog_button1 = gtk_button_new_with_label ("OK");
  gtk_widget_set_name (msg_dialog_button1, "msg_dialog_button1");
  gtk_widget_ref (msg_dialog_button1);
  gtk_object_set_data_full (GTK_OBJECT (msg_dialog1), "msg_dialog_button1", msg_dialog_button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (msg_dialog_button1);
  gtk_box_pack_start (GTK_BOX (dialog_action_area3), msg_dialog_button1, FALSE, FALSE, 0);

  gtk_signal_connect (GTK_OBJECT (msg_dialog1), "delete_event",
                      GTK_SIGNAL_FUNC (on_msg_dialog1_delete_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (msg_dialog_button1), "clicked",
                      GTK_SIGNAL_FUNC (on_msg_dialog_button1_clicked),
                      msg_dialog1);

  return msg_dialog1;
}


void
on_msg_dialog_button1_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(user_data);
}


gboolean
on_msg_dialog1_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	gtk_widget_destroy(widget);
  return FALSE;
}


/* gfx window */

GtkWidget* create_my_gfx_window1 (struct GFX_DATA *gfx, char *gtk_descr, char *window_title)
{
  GtkWidget *gfx_window1;
  GtkWidget *drawingarea1 = NULL;

  gfx_window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (gfx_window1, gtk_descr);
  gtk_object_set_data (GTK_OBJECT (gfx_window1), gtk_descr, gfx_window1);
  gtk_window_set_title (GTK_WINDOW (gfx_window1), window_title);
  gtk_window_set_policy (GTK_WINDOW (gfx_window1), FALSE, FALSE, TRUE);

  drawingarea1 = gtk_drawing_area_new ();
  gtk_widget_set_name (drawingarea1, "drawingarea1");
  gtk_widget_ref (drawingarea1);
  gtk_object_set_data_full (GTK_OBJECT (gfx_window1), "drawingarea1", drawingarea1,
                           (GtkDestroyNotify) gtk_widget_unref);

  gtk_drawing_area_size (GTK_DRAWING_AREA (drawingarea1), gfx->Width, gfx->Height);

  gtk_container_add (GTK_CONTAINER (gfx_window1), drawingarea1);  // flipped up
  gtk_signal_connect (GTK_OBJECT (drawingarea1), "expose-event",
       	              GTK_SIGNAL_FUNC (on_darea_expose), gfx);

  gtk_widget_show (drawingarea1);

  gtk_widget_set_events (drawingarea1, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_LEAVE_NOTIFY_MASK);	

  gtk_signal_connect (GTK_OBJECT (gfx_window1), "delete_event",
                      GTK_SIGNAL_FUNC (on_gfx_window1_delete_event),
       	              NULL);

  return gfx_window1;
}


GtkWidget* create_my_tex_align_window1 (struct GFX_DATA *gfx_tex_to_merge[3], char *gtk_descr, char *window_title)
{
  GtkWidget *gfx_window1;
  GtkWidget *drawingarea1 = NULL;

  gfx_window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (gfx_window1, gtk_descr);
  gtk_object_set_data (GTK_OBJECT (gfx_window1), gtk_descr, gfx_window1);
  gtk_window_set_title (GTK_WINDOW (gfx_window1), window_title);
  gtk_window_set_policy (GTK_WINDOW (gfx_window1), FALSE, FALSE, TRUE);

  drawingarea1 = gtk_drawing_area_new ();
  gtk_widget_set_name (drawingarea1, "tex_align_drawingarea1");
  gtk_widget_ref (drawingarea1);
  gtk_object_set_data_full (GTK_OBJECT (gfx_window1), "tex_align_drawingarea1", drawingarea1,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_drawing_area_size (GTK_DRAWING_AREA (drawingarea1), gfx_tex_to_merge[1]->Width, gfx_tex_to_merge[1]->Height);

  gtk_container_add (GTK_CONTAINER (gfx_window1), drawingarea1);  // flipped up
  gtk_signal_connect (GTK_OBJECT (drawingarea1), "expose-event",
       	              GTK_SIGNAL_FUNC (on_darea_expose), gfx_tex_to_merge[0]);

  gtk_widget_show (drawingarea1);


  gtk_signal_connect (GTK_OBJECT (drawingarea1), "button_press_event",
                      GTK_SIGNAL_FUNC (on_tex_align_drawingarea1_button_press_event),
                      gfx_tex_to_merge);

  gtk_widget_set_events (drawingarea1, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_LEAVE_NOTIFY_MASK);	

  gtk_signal_connect (GTK_OBJECT (gfx_window1), "delete_event",
                      GTK_SIGNAL_FUNC (on_gfx_window1_delete_event),
       	              NULL);

  return gfx_window1;
}


gboolean
on_tex_align_drawingarea1_button_press_event (GtkWidget       *widget,
                                              GdkEventButton  *event,
                                              gpointer         user_data)
{
	int x = 0, y, r = 0, g = 0, b = 0;
	GtkWidget *tex_align_drawingarea1;

	if(!event) {
		if(gfx_tex_to_merge[0]->Data)
			free(gfx_tex_to_merge[0]->Data);
		gfx_tex_to_merge[0]->Width = gfx_tex_to_merge[1]->Width;
		gfx_tex_to_merge[0]->Height = gfx_tex_to_merge[1]->Height;
		gfx_tex_to_merge[0]->Data = (int*) malloc(gfx_tex_to_merge[0]->Width * gfx_tex_to_merge[0]->Height * sizeof(int));
		memcpy(gfx_tex_to_merge[0]->Data, gfx_tex_to_merge[1]->Data, gfx_tex_to_merge[0]->Width * gfx_tex_to_merge[0]->Height * sizeof(int));
	} else if(event->button == 1) {
			Renderer_Params.StartX = event->x;
			Renderer_Params.StartY = event->y;
			if(Renderer_Params.StartX > gfx_tex_to_merge[0]->Width - gfx_tex_to_merge[2]->Width - 1)
				Renderer_Params.StartY = gfx_tex_to_merge[0]->Width - gfx_tex_to_merge[2]->Width - 1;
			while(Renderer_Params.StartY > gfx_tex_to_merge[2]->Height)
				Renderer_Params.StartY -= gfx_tex_to_merge[2]->Height;
		}
	if(tex_align_window1) {
		if(Renderer_Params.StartX > gfx_tex_to_merge[0]->Width - gfx_tex_to_merge[2]->Width - 1)
			Renderer_Params.StartX = gfx_tex_to_merge[0]->Width - gfx_tex_to_merge[2]->Width - 1;
		while(Renderer_Params.StartY > gfx_tex_to_merge[2]->Height)
			Renderer_Params.StartY -= gfx_tex_to_merge[2]->Height;
		memcpy(gfx_tex_to_merge[0]->Data, gfx_tex_to_merge[1]->Data, gfx_tex_to_merge[0]->Width * gfx_tex_to_merge[0]->Height * sizeof(int));
		for(y = 0; y < gfx_tex_to_merge[0]->Height; y++)
			for(x = 0; x < gfx_tex_to_merge[2]->Width; x++) {
				//*(gfx_tex_to_merge[0]->Data + (y * gfx_tex_to_merge[0]->Width) + Renderer_Params.StartX + x) = ((*(gfx_tex_to_merge[0]->Data + (y * gfx_tex_to_merge[0]->Width) + Renderer_Params.StartX + x)) >> 1) + ((*(gfx_tex_to_merge[2]->Data + (((y + gfx_tex_to_merge[2]->Height - Renderer_Params.StartY) % gfx_tex_to_merge[2]->Height) * gfx_tex_to_merge[2]->Width) + x)) >> 1);
				r = ((( (*(gfx_tex_to_merge[0]->Data + (y * gfx_tex_to_merge[0]->Width) + Renderer_Params.StartX + x))     ) & 255) + (( (*(gfx_tex_to_merge[2]->Data + (((y + gfx_tex_to_merge[2]->Height - Renderer_Params.StartY) % gfx_tex_to_merge[2]->Height) * gfx_tex_to_merge[2]->Width) + x))     ))) >> 1;
				g = ((( (*(gfx_tex_to_merge[0]->Data + (y * gfx_tex_to_merge[0]->Width) + Renderer_Params.StartX + x)) >> 8) & 255) + (( (*(gfx_tex_to_merge[2]->Data + (((y + gfx_tex_to_merge[2]->Height - Renderer_Params.StartY) % gfx_tex_to_merge[2]->Height) * gfx_tex_to_merge[2]->Width) + x)) >> 8) & 255)) >> 1;
				b = ((( (*(gfx_tex_to_merge[0]->Data + (y * gfx_tex_to_merge[0]->Width) + Renderer_Params.StartX + x)) >>16) & 255) + (( (*(gfx_tex_to_merge[2]->Data + (((y + gfx_tex_to_merge[2]->Height - Renderer_Params.StartY) % gfx_tex_to_merge[2]->Height) * gfx_tex_to_merge[2]->Width) + x)) >>16) & 255)) >> 1;
				*(gfx_tex_to_merge[0]->Data + (y * gfx_tex_to_merge[0]->Width) + Renderer_Params.StartX + x) =  (r + (g << 8) + (b << 16));
			}
			//memcpy(gfx_tex_to_merge[0]->Data + (y * gfx_tex_to_merge[0]->Width) + Renderer_Params.StartX, gfx_tex_to_merge[2]->Data + (((y + gfx_tex_to_merge[0]->Height - Renderer_Params.StartY) % gfx_tex_to_merge[2]->Height) * gfx_tex_to_merge[2]->Width), gfx_tex_to_merge[2]->Width * sizeof(int));

		tex_align_drawingarea1 = lookup_widget (GTK_WIDGET (tex_align_window1), "tex_align_drawingarea1");
		on_darea_expose(tex_align_drawingarea1, NULL, gfx_tex_to_merge[0]);
	}

  return FALSE;
}


gboolean
on_darea_expose (GtkWidget *widget,
                 GdkEventExpose *event,
                 struct GFX_DATA *gfx)
{
#if __BYTE_ORDER == __BIG_ENDIAN   
  guchar * ptr;
  int p, z, u;
  ptr = (guchar *)malloc(4*gfx->Width*gfx->Height);
  u = -1;
  for (z = 0; z < gfx->Width*gfx->Height; z++) {
    p = gfx->Data[z];
    ptr[++u] = p&255;
    ptr[++u] = (p>>8)&255;     
    ptr[++u] = (p>>16)&255;
    ptr[++u] = 0;
  }  
  gdk_draw_rgb_32_image (widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
                         0, 0, gfx->Width, gfx->Height,
                         GDK_RGB_DITHER_MAX, ptr, gfx->Width*4);
  free(ptr);
#else
  gdk_draw_rgb_32_image (widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
                         0, 0, gfx->Width, gfx->Height,
                         GDK_RGB_DITHER_MAX, (guchar*)gfx->Data, gfx->Width*4);
#endif
  return 0;
}


/* resize dialog */

int widthmax = 120;

GtkWidget*
create_my_resize_window1 (int width)
{
  GtkWidget *resize_window1;
  GtkWidget *vbox7;
  GtkWidget *hbox6;
  GtkWidget *hbox7;   
  GtkObject *newwidth_spinbutton_adj;
  GtkWidget *newwidth_spinbutton;
  GtkObject *maxwidth_spinbutton_adj;
  GtkWidget *maxwidth_spinbutton;
  GtkWidget *label12;
  GtkWidget *label13;   
  guint resize_cut_button_key;
  GtkWidget *resize_cut_button;
  GtkWidget *resize_close_button;   
  GtkAccelGroup *accel_group;

  accel_group = gtk_accel_group_new ();

  resize_window1 = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_widget_set_name (resize_window1, "resize_window1");
  gtk_object_set_data (GTK_OBJECT (resize_window1), "resize_window1", resize_window1);
  gtk_window_set_title (GTK_WINDOW (resize_window1), "resize texture");
  gtk_window_set_policy (GTK_WINDOW (resize_window1), FALSE, FALSE, FALSE);

  vbox7 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox7, "vbox7");
  gtk_widget_ref (vbox7);
  gtk_object_set_data_full (GTK_OBJECT (resize_window1), "vbox7", vbox7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox7);
  gtk_container_add (GTK_CONTAINER (resize_window1), vbox7);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbox6, "hbox6");
  gtk_widget_ref (hbox6);
  gtk_object_set_data_full (GTK_OBJECT (resize_window1), "hbox6", hbox6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox6);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox6, TRUE, TRUE, 0);

  newwidth_spinbutton_adj = gtk_adjustment_new (width, 2, width, 1, 10, 10);
  newwidth_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (newwidth_spinbutton_adj), 1, 0);
  gtk_widget_set_name (newwidth_spinbutton, "newwidth_spinbutton");
  gtk_widget_ref (newwidth_spinbutton);
  gtk_object_set_data_full (GTK_OBJECT (resize_window1), "newwidth_spinbutton", newwidth_spinbutton,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (newwidth_spinbutton);
  gtk_box_pack_start (GTK_BOX (hbox6), newwidth_spinbutton, TRUE, TRUE, 4);
  gtk_widget_set_usize (newwidth_spinbutton, 65, -2);

  label12 = gtk_label_new ("new width");
  gtk_widget_set_name (label12, "label12");
  gtk_widget_ref (label12);
  gtk_object_set_data_full (GTK_OBJECT (resize_window1), "label12", label12,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label12);
  gtk_box_pack_start (GTK_BOX (hbox6), label12, TRUE, TRUE, 4);
  gtk_label_set_justify (GTK_LABEL (label12), GTK_JUSTIFY_LEFT);

  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbox7, "hbox7");
  gtk_widget_ref (hbox7);
  gtk_object_set_data_full (GTK_OBJECT (resize_window1), "hbox7", hbox7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox7, FALSE, TRUE, 0);

  maxwidth_spinbutton_adj = gtk_adjustment_new (widthmax, 2, 2000, 1, 10, 10);
  maxwidth_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (maxwidth_spinbutton_adj), 1, 0);
  gtk_widget_set_name (maxwidth_spinbutton, "maxwidth_spinbutton");
  gtk_widget_ref (maxwidth_spinbutton);
  gtk_object_set_data_full (GTK_OBJECT (resize_window1), "maxwidth_spinbutton", maxwidth_spinbutton,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (maxwidth_spinbutton);
  gtk_box_pack_start (GTK_BOX (hbox7), maxwidth_spinbutton, TRUE, TRUE, 4);
  gtk_widget_set_usize (maxwidth_spinbutton, 65, -2);

  label13 = gtk_label_new ("max width");
  gtk_widget_set_name (label13, "label13");
  gtk_widget_ref (label13);
  gtk_object_set_data_full (GTK_OBJECT (resize_window1), "label13", label13,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label13);
  gtk_box_pack_start (GTK_BOX (hbox7), label13, TRUE, TRUE, 4);
  gtk_label_set_justify (GTK_LABEL (label13), GTK_JUSTIFY_LEFT);
   

  resize_cut_button = gtk_button_new_with_label ("");
  resize_cut_button_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (resize_cut_button)->child),
                                   "_resize");
  gtk_widget_add_accelerator (resize_cut_button, "clicked", accel_group,
                              resize_cut_button_key, GDK_MOD1_MASK, 0);
  gtk_widget_set_name (resize_cut_button, "resize_cut_button");
  gtk_widget_ref (resize_cut_button);
  gtk_object_set_data_full (GTK_OBJECT (resize_window1), "resize_cut_button", resize_cut_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (resize_cut_button);
  gtk_box_pack_start (GTK_BOX (vbox7), resize_cut_button, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (resize_cut_button), 4);

  resize_close_button = gtk_button_new_with_label ("close");
  gtk_widget_set_name (resize_close_button, "resize_close_button");
  gtk_widget_ref (resize_close_button);
  gtk_object_set_data_full (GTK_OBJECT (resize_window1), "resize_close_button", resize_close_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (resize_close_button);
  gtk_box_pack_start (GTK_BOX (vbox7), resize_close_button, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (resize_close_button), 4);

  gtk_signal_connect (GTK_OBJECT (resize_window1), "delete_event",
                      GTK_SIGNAL_FUNC (on_resize_window1_delete_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (resize_cut_button), "clicked",
                      GTK_SIGNAL_FUNC (on_resize_cut_button_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (resize_close_button), "clicked",
                      GTK_SIGNAL_FUNC (on_resize_window1_delete_event),
                      NULL);

  gtk_window_add_accel_group (GTK_WINDOW (resize_window1), accel_group);

  return resize_window1;
}




void
on_main_menu_file1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_main_menu_base1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_main_menu_texture1_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_main_menu_stereogram1_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_main_menu_info1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_main_window1_destroy                (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_main_quit();
}



/* preventing file dialog exits */
gboolean
on_base_open_dialogue1_delete_event    (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_widget_hide (widget);
  return TRUE;
}


gboolean
on_texture_open_dialogue1_delete_event (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_widget_hide (widget);
  return TRUE;
}


gboolean
on_stereogram_save_dialogue1_delete_event
                                        (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_widget_hide (widget);
  return TRUE;
}

void
load_base()
{     	
        Renderer_Params.StartX = 0;
	Renderer_Params.StartY = 0;
	/* destroy former mem allocs */
	if(GFX_Base.Data) {
		free(GFX_Base.Data);
		GFX_Base.Data = NULL;
	}

	/* read gfx data from file and display it */
	if(!GFX_Read_File(base_file_name, &GFX_Base)) {
		/* base_gfx_window1 = create_my_gfx_window1 (&GFX_Base, "base_gfx_window1", "base image"); */
		/* gtk_widget_show (base_gfx_window1); */
	} else {
		GtkWidget* dialog;
		// error reading gfx data
		if(GFX_Base.Data) {
			free(GFX_Base.Data);
			GFX_Base.Data = NULL;
		}
		base_file_name = NULL;
		dialog = create_my_msg_dialog1 ("GFX IO Error", "\nCould not read base image from file,\nunsupported format.\n");
		gtk_widget_show(dialog);
	}   
}

void
load_extern_base(char *file)
{
        base_file_name = file;
        if (!base_open_dialogue1) 
          base_open_dialogue1 = create_base_open_dialogue1();
        gtk_file_selection_set_filename((GtkFileSelection*)base_open_dialogue1, base_file_name);
        load_base();
}

/* managing graphics dialogs */

void
on_ok_button1_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  base_file_name = gtk_file_selection_get_filename(GTK_FILE_SELECTION (base_open_dialogue1));
  gtk_widget_hide (base_open_dialogue1);

	/* destroy former image view */
	if(base_gfx_window1) {
		gtk_widget_destroy(base_gfx_window1);
		base_gfx_window1 = NULL;
	}
	if(tex_align_window1) {
		gtk_widget_destroy(tex_align_window1);
		tex_align_window1 = NULL;
	}
        load_base();
}


void
on_cancel_button1_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (base_open_dialogue1);
}

void
load_texture()
{
	Renderer_Params.StartX = 0;
	Renderer_Params.StartY = 0;
	if(resize_window1) {
		gtk_widget_destroy (resize_window1);
		resize_window1 = NULL;
	}
	if(generate_window1)
		gtk_widget_hide(generate_window1);
	/* destroy former mem allocs */
	if(GFX_Texture.Data) {
		free(GFX_Texture.Data);
		GFX_Texture.Data = NULL;
	}

	/* read gfx data from file and display it */
	if(!GFX_Read_File(texture_file_name,  &GFX_Texture )) {
                GFX_Resize(&GFX_Texture , widthmax);   
		/* texture_gfx_window1 = create_my_gfx_window1 ( &GFX_Texture , "texture_gfx_window1", "texture image"); */
		/* gtk_widget_show (texture_gfx_window1); */
	} else {
		// error reading gfx data
		GtkWidget *dialog;
		if(GFX_Texture.Data) {
			free(GFX_Texture.Data);
			GFX_Texture.Data = NULL;
		}
		texture_file_name = NULL;
		dialog = create_my_msg_dialog1 ("GFX IO Error", "\nCould not read texture image from file,\nunsupported format.\n");
		gtk_widget_show(dialog);
	}   
}

void
load_extern_texture(char *file)
{
        texture_file_name = file;
        if (!texture_open_dialogue1)
          texture_open_dialogue1 = create_texture_open_dialogue1();
        gtk_file_selection_set_filename((GtkFileSelection*)texture_open_dialogue1, texture_file_name);
        load_texture();
}

void
on_ok_button2_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  texture_file_name = gtk_file_selection_get_filename(GTK_FILE_SELECTION (texture_open_dialogue1));
  gtk_widget_hide (texture_open_dialogue1);

	/* destroy former image view */
	if(texture_gfx_window1) {
		gtk_widget_destroy(texture_gfx_window1);
		texture_gfx_window1 = NULL;
	}
	if(tex_align_window1) {
		gtk_widget_destroy(tex_align_window1);
		tex_align_window1 = NULL;
	}
        load_texture();
}


void
on_cancel_button2_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (texture_open_dialogue1);
}


void
on_ok_button3_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *option_menu, *menu, *active_item, *dialog;

	stereo_file_name = gtk_file_selection_get_filename(GTK_FILE_SELECTION (stereogram_save_dialogue1));
	gtk_widget_hide (stereogram_save_dialogue1);

	/* getting output file type */
	option_menu = lookup_widget (GTK_WIDGET (main_window1), "save_type_menu");
	menu = GTK_OPTION_MENU (option_menu)->menu;
	active_item = gtk_menu_get_active(GTK_MENU (menu));
	stereo_gfx = g_list_index (GTK_MENU_SHELL (menu)->children, active_item);

	if(!GFX_Write_File(stereo_file_name, stereo_gfx+1,  &GFX_Stereo )) {
		printf("gfx data saved successfully;\n");
		dialog = create_my_msg_dialog1 ("Stereogram", "\nStereogram saved successfully.\n");
		gtk_widget_show(dialog);
	} else {
		printf("saving gfx data FAILED;\n");
		dialog = create_my_msg_dialog1 ("GFX IO Error", "\nCould not save stereogram.\nPerhaps not enough free disk space left?\n");
		gtk_widget_show(dialog);
	}
}


void
on_cancel_button3_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (stereogram_save_dialogue1);
}


gboolean
on_info_dialog1_delete_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_widget_hide (widget);
  return TRUE;
}


void
on_info_ok_button1_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (info_dialog1);
}


gboolean
on_info_dialog2_delete_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_widget_hide (widget);
  return TRUE;
}


void
on_info_ok_button2_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (info_dialog2);
}


gboolean
on_info_dialog3_delete_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_widget_hide (widget);
  return TRUE;
}


void
on_info_ok_button3_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (info_dialog3);
}


gboolean
on_gfx_window1_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_widget_hide (widget);
  return TRUE;
}


void
on_main_menu_quit_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_main_quit();
}


void
on_baseloadmenu_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_base_load_button_clicked(NULL, NULL);
}

void
on_baserandommenu_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_base_random_button_clicked(NULL, NULL);
}

void
on_baseeditmenu_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_base_edit_button_clicked(NULL, NULL);
}


void
on_baseshowmenu_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_base_show_button_clicked(NULL, NULL);
}


void
on_baseinvertmenu_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_base_invert_button_clicked (NULL, NULL);
}


void
on_textureloadmenu_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_texture_load_button_clicked(NULL, NULL);
}


void
on_texturegeneratemenu_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_texture_generate_button_clicked(NULL, NULL);
}


void
on_textureshowmenu_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_texture_show_button_clicked(NULL, NULL);
}


void
on_texturealignmentmenu_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_texture_alignment_button_clicked(NULL, NULL);
}


void
on_texturecutmenu_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_stereogramrendermenu_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_render_button_clicked(NULL, NULL);
}


void
on_stereogramshowmenu_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_stereogram_show_button_clicked(NULL, NULL);
}


void
on_stereogramsavemenu_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_stereogram_save_button_clicked(NULL, NULL);
}


void
on_aboutmenu_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if(!info_dialog1)
		info_dialog1 = create_info_dialog1 ();
	gtk_widget_show (info_dialog1);
	gtk_widget_activate (info_dialog1);
}

void
on_physicsmenu_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
        GtkWidget *drawingarea2;
        static struct GFX_DATA gfx;
        gchar ge[256];   
	if(!info_dialog2) {
	  info_dialog2 = create_info_dialog2 ();
	  drawingarea2 = lookup_widget (GTK_WIDGET (info_dialog2), "drawingarea2");
	  sprintf(ge, "%s/doc/stereo_dia.png", PACKAGE_DATA_DIR);
          if(!GFX_Read_File(ge, &gfx)) {
            gtk_widget_set_usize (drawingarea2, gfx.Width, gfx.Height);
	    gtk_signal_connect (GTK_OBJECT (drawingarea2), "expose-event",
         	              GTK_SIGNAL_FUNC (on_darea_expose), &gfx);
	  }
	}
 	gtk_widget_show (info_dialog2);
	gtk_widget_activate (info_dialog2);
}

void
on_helpmenu_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
        GtkWidget *text1;
        FILE *fd;
        gchar ge[1050];
   
	if(!info_dialog3) {
	  info_dialog3 = create_info_dialog3 ();
	  text1 = lookup_widget (GTK_WIDGET (info_dialog3), "text1");
	  sprintf(ge, "%s/doc/help", PACKAGE_DATA_DIR);
          fd = fopen(ge, "r");
	  while(fd && fgets(ge, 1024, fd)!=NULL) {
             gtk_text_insert (GTK_TEXT (text1), NULL, NULL, NULL,
		       ge, strlen (ge));
          }
          if (fd) 
	     fclose(fd);
	  else {
	     gtk_text_insert (GTK_TEXT (text1), NULL, NULL, NULL,
		       "!!! Couldn't find\n\n", 19);
             gtk_text_insert (GTK_TEXT (text1), NULL, NULL, NULL,
		       ge, strlen (ge));
	  }
          gtk_text_set_editable (GTK_TEXT (text1), FALSE);   
	}
	   
	gtk_widget_show (info_dialog3);
	gtk_widget_activate (info_dialog3);   
}

void
on_base_load_button_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	if(!base_open_dialogue1)
		base_open_dialogue1 = create_base_open_dialogue1 ();
        if (base_editor1)
                gtk_widget_hide (base_editor1);   
	gtk_widget_show (base_open_dialogue1);
	gtk_widget_activate (base_open_dialogue1);
}

void Randomize(char * name_dir, char *output)
{
struct dirent *file;
DIR *directory;
unsigned long int count, rand;
int i;

   if (!name_dir || !*name_dir || !strcmp(name_dir, "-")) return;
   
   directory = opendir(name_dir);
   if (!directory) return;
   count = 0;
   while((file = readdir(directory)) != NULL)
       if (strcmp(file->d_name, ".") && strcmp(file->d_name, "..")) count++;
   closedir(directory);

   if (count==0) {
       strcpy(output, PACKAGE_DATA_DIR);
       return;
   }
 
   rand = 
     ((unsigned long int)random() + (unsigned long int) time(NULL)) % count;
   
   i = 0;
   directory = opendir(name_dir);
   while((file = readdir(directory)) != NULL)
   {
      if (!strcmp(file->d_name, ".") || !strcmp(file->d_name, ".."))
	 continue;
      if (i == rand) {
         sprintf(output, "%s/%s", name_dir, file->d_name);
	 break;
      }
      i++;      
   }
   closedir(directory);
}

void
on_base_random_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
char direct[512];
static char output[512];
   
        if (base_editor1)
          gtk_widget_hide (base_editor1);
        
        if (GFX_Base.Data && base_gfx_window1)
          gtk_widget_hide(base_gfx_window1);
      
        sprintf(direct, "%s/base", PACKAGE_DATA_DIR);
        Randomize(direct, output);
        strcpy(direct, output);
        Randomize(direct, output);
        base_file_name = output;

        if (!base_open_dialogue1) 
          base_open_dialogue1 = create_base_open_dialogue1();
        gtk_file_selection_set_filename((GtkFileSelection*)base_open_dialogue1, base_file_name);
        load_base();   
}

void
on_base_edit_button_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
        char *ptr;
        int l;
   
        if (base_file_name && (l=strlen(base_file_name))>2 &&
	    !strcmp(base_file_name+l-2, ".c")) 
          ptr = base_file_name;
        else
          ptr = "Untitled";
        if (!base_editor1)
          base_editor1 = GTK_editor(ptr, (void *)load_extern_base);
        else {
          gtk_widget_show(base_editor1);
	  GTK_editor_set_file(ptr);	   
	}
}

void
on_base_show_button_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
   
        if(base_gfx_window1) {
		gtk_widget_destroy(base_gfx_window1);
	   	base_gfx_window1 = NULL;
	}
    
	if(GFX_Base.Data) {
		base_gfx_window1 = create_my_gfx_window1 ( &GFX_Base , "base_gfx_window1", "base image");
		gtk_widget_show (base_gfx_window1);
	} else
		if(!GFX_Base.Data)
			on_baseloadmenu_activate(NULL, NULL);
		else if(base_gfx_window1) {
			gtk_widget_show (base_gfx_window1);
			gtk_widget_activate(base_gfx_window1);
		}
}


void
on_texture_load_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	if(generate_window1)
		gtk_widget_hide(generate_window1);
        if (texture_editor1)
                gtk_widget_hide (texture_editor1);   
	if(!texture_open_dialogue1)
		texture_open_dialogue1 = create_texture_open_dialogue1 ();
	gtk_widget_show (texture_open_dialogue1);
	gtk_widget_activate (texture_open_dialogue1);
}


void
on_texture_random_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
char direct[512];
static char output[512];

        if (texture_editor1)
          gtk_widget_hide (texture_editor1);
        
        if (GFX_Texture.Data && texture_gfx_window1)
          gtk_widget_hide(texture_gfx_window1);
   
        sprintf(direct, "%s/textures", PACKAGE_DATA_DIR);
        Randomize(direct, output);
        strcpy(direct, output);
        Randomize(direct, output);
        texture_file_name = output;

        if (!texture_open_dialogue1) 
          texture_open_dialogue1 = create_texture_open_dialogue1();
        gtk_file_selection_set_filename((GtkFileSelection*)texture_open_dialogue1, texture_file_name);
        load_texture();
}

void
on_texture_generate_button_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{
	if(!generate_window1)
		generate_window1 = create_generate_window1 ();
	gtk_widget_show (generate_window1);
	gtk_widget_activate (generate_window1);
}


void
on_texture_show_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
        if(texture_gfx_window1) {
		gtk_widget_destroy(texture_gfx_window1);
	   	texture_gfx_window1 = NULL;
	}
   
	if(GFX_Texture.Data) {
		texture_gfx_window1 = create_my_gfx_window1 ( &GFX_Texture , "texture_gfx_window1", "texture image");
		gtk_widget_show (texture_gfx_window1);
	} else
		if(!GFX_Texture.Data)
			on_textureloadmenu_activate(NULL, NULL);
		else if(texture_gfx_window1) {
			gtk_widget_show (texture_gfx_window1);
			gtk_widget_activate(texture_gfx_window1);
		}
}


void
on_texture_alignment_button_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
	if(!tex_align_window1) {
		if(GFX_Base.Data && GFX_Texture.Data) {
			if(GFX_Base.Width > GFX_Texture.Width) {
				gfx_tex_to_merge[0] = &tex_merge;
				gfx_tex_to_merge[1] =  &GFX_Base ;
				gfx_tex_to_merge[2] =  &GFX_Texture ;
				tex_align_window1 = create_my_tex_align_window1 (gfx_tex_to_merge, "tex_align_window1", "texture alignment");
				tex_merge.Data = NULL;
			} else {
				GtkWidget *dialog;
				dialog = create_my_msg_dialog1 ("Alignment Error", "\nThe texture width must be less than the width of the base.\nPlease load another texture or choose a greater base image.\n");
				gtk_widget_show(dialog);
				printf("texture width must be less than the width of the base file.\n");
			}
		} else {
			GtkWidget *dialog;
			dialog = create_my_msg_dialog1 ("Alignment Error", "\nNo texture-base pair present.\n\nYou must load a base and a texture.\n");
			gtk_widget_show(dialog);
			printf("no texture/base file pair present.\n");
		}
	}
	if(tex_align_window1) {
		gtk_widget_show(tex_align_window1);
		gtk_widget_activate(tex_align_window1);
		on_tex_align_drawingarea1_button_press_event (NULL, NULL, NULL);
	}
}


void
on_texture_cut_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	if(GFX_Texture.Data) {
		if(!resize_window1)
			resize_window1 = create_my_resize_window1 (GFX_Texture.Width);
		gtk_widget_show(resize_window1);
		gtk_widget_activate(resize_window1);
	} else {
		GtkWidget *dialog;
		dialog = create_my_msg_dialog1 ("resize Error", "\nNo texture present.\n\nYou must first of all define a texture.\n");
		gtk_widget_show(dialog);
	}
}


void
on_render_button_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
		if(GFX_Base.Data && GFX_Texture.Data) {
			// get latest params
			GtkWidget *editable;

			editable = lookup_widget (GTK_WIDGET (main_window1), "aa_spinbutton");
			Renderer_Params.AA = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (editable));

			editable = lookup_widget (GTK_WIDGET (main_window1), "zoom_spinbutton");
			Renderer_Params.Zoom = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (editable));

			editable = lookup_widget (GTK_WIDGET (main_window1), "distance_spinbutton");
			Renderer_Params.Distance = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON (editable));

			editable = lookup_widget (GTK_WIDGET (main_window1), "eyeshift_spinbutton");
			Renderer_Params.Eyeshift = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON (editable));

			editable = lookup_widget (GTK_WIDGET (main_window1), "aar_checkbutton");
			if (GTK_TOGGLE_BUTTON (editable)->active)
				Renderer_Params.AAr = 1;
			else
				Renderer_Params.AAr = 0;

			editable = lookup_widget (GTK_WIDGET (main_window1), "linearrendering_checkbutton");
			if (GTK_TOGGLE_BUTTON (editable)->active)
				Renderer_Params.Linear = 0;
			else
				Renderer_Params.Linear = 1;

			editable = lookup_widget (GTK_WIDGET (main_window1), "triangle_checkbutton");
			if (GTK_TOGGLE_BUTTON (editable)->active)
				Stereo_Triangles = 1;
			else
				Stereo_Triangles = 0;

			// clear up all used mem from before and kill window.
			if(stereo_gfx_window1) {
				gtk_widget_destroy (stereo_gfx_window1);
				stereo_gfx_window1 = NULL;
			}
			if(GFX_Stereo.Data)
				Renderer_Close( &Renderer_Data );
			
                        // call the renderer
			if(!Renderer_Initialize( &Renderer_Data, &Renderer_Params )) {
				int y;
				//printf("renderer initialized\n");
				for(y = 0; (y < GFX_Base.Height); y++) {
				        //printf("rendering line %i...\n", y);
					Renderer_ProcessLine(y, &Renderer_Data);
				}
				//printf("...rendering done;\n");
				if(Stereo_Triangles)
					GFX_AddTriangles(GFX_Stereo.Width / 2, 20 * Renderer_Params.Zoom, GFX_Texture.Width * Renderer_Params.Zoom,  &GFX_Stereo );
				stereo_gfx_window1 = create_my_gfx_window1 ( &GFX_Stereo , "stereo_gfx_window1", "stereogram");
				gtk_widget_show (stereo_gfx_window1);
			} else {
				GtkWidget *dialog;
				printf("initilizing renderer FAILED;\n");
				if(GFX_Base.Width <= GFX_Texture.Width)
					dialog = create_my_msg_dialog1 ("Rendering Error", "\nUnable to initialize renderer.\n\nReason:\n  The width of the texture must be smaller than the width of the base image.\nPlease load another texture or choose a greater base image.\n");
				else
					dialog = create_my_msg_dialog1 ("Rendering Error", "\nUnable to initialize renderer.\n\nPossible reasons:\n  not enough free memory\n  illegal parametre\n  internal renderer inconsistence\n\nIf this error occurs frequently, please report this to the author.\n");
				gtk_widget_show(dialog);
			}
		} else {
			GtkWidget *dialog;
			/* illegal parametre, drop note */
			Renderer_Close( &Renderer_Data );
			dialog = create_my_msg_dialog1 ("Renderer Error", "\nYou must load a base and a texture file.\n");
			gtk_widget_show(dialog);
		}
}


void
on_stereogram_save_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	if(GFX_Stereo.Data) {
		if(!stereogram_save_dialogue1)
			stereogram_save_dialogue1 = create_stereogram_save_dialogue1 ();
		gtk_widget_show (stereogram_save_dialogue1);
		gtk_widget_activate (stereogram_save_dialogue1);
	} else {
		GtkWidget *dialog;
		dialog = create_my_msg_dialog1 ("Stereogram saving Error", "\nYou must render a stereogram before saving it in release 0.12a and above.\n");
		gtk_widget_show(dialog);
	}
}


void
on_stereogram_show_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	if(!stereo_gfx_window1 && GFX_Stereo.Data) {
		stereo_gfx_window1 = create_my_gfx_window1 ( &GFX_Stereo , "stereo_gfx_window1", "stereogram");
		gtk_widget_show (stereo_gfx_window1);
	} else if(stereo_gfx_window1 && GFX_Stereo.Data) {
		gtk_widget_show (stereo_gfx_window1);
		gtk_widget_activate(stereo_gfx_window1);
	}
}


gboolean
on_generate_window1_delete_event       (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_widget_hide (generate_window1);
  return TRUE;
}


void
on_generate_random_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	int height, width, type;
	GtkWidget *editable;

	texture_file_name = NULL;
	if(texture_open_dialogue1)
		gtk_widget_hide (texture_open_dialogue1);

        if (texture_editor1)
                gtk_widget_hide (texture_editor1);   
   
	/* destroy former image view */
	if(texture_gfx_window1) {
		gtk_widget_destroy(texture_gfx_window1);
		texture_gfx_window1 = NULL;
	}
	if(tex_align_window1) {
		gtk_widget_destroy(tex_align_window1);
		tex_align_window1 = NULL;
	}
	Renderer_Params.StartX = 0;
	Renderer_Params.StartY = 0;
	if(resize_window1) {
		gtk_widget_destroy (resize_window1);
		resize_window1 = NULL;
	}
	/* destroy former mem allocs */
	if(GFX_Texture.Data) {
		free(GFX_Texture.Data);
		GFX_Texture.Data = NULL;
	}

	editable = lookup_widget (GTK_WIDGET (generate_window1), "generate_width_spinbutton");
	width = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (editable));
	editable = lookup_widget (GTK_WIDGET (generate_window1), "generate_height_spinbutton");
	height = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (editable));

	editable = lookup_widget (GTK_WIDGET (generate_window1), "generate_color_radiobutton");
	if (GTK_TOGGLE_BUTTON (editable)->active)
		type = GFX_TEX_COLORED;
	else {
		editable = lookup_widget (GTK_WIDGET (generate_window1), "generate_grayscale_radiobutton");
		if (GTK_TOGGLE_BUTTON (editable)->active)
			type = GFX_TEX_GRAYSCALE;
		else {
			editable = lookup_widget (GTK_WIDGET (generate_window1), "generate_sinline_radiobutton");
			if (GTK_TOGGLE_BUTTON (editable)->active)
				type = GFX_TEX_SINLINE;
			else
				type = GFX_TEX_BW;
		}
	}
	/* read gfx data from file and display it */
	if(!GFX_Generate_RandomTexture( &GFX_Texture , width, height, type)) {
		/* texture_gfx_window1 = create_my_gfx_window1 ( &GFX_Texture , "texture_gfx_window1", "texture image"); */
		/* gtk_widget_show (texture_gfx_window1); */
	} else {
		// error reading gfx data
		GtkWidget *dialog;
		if(GFX_Texture.Data) {
			free(GFX_Texture.Data);
			GFX_Texture.Data = NULL;
		}
		dialog = create_my_msg_dialog1 ("GFX IO Error", "\nCould not create random texture image,\nperhaps not enought memory available.\n");
		gtk_widget_show(dialog);
	}
}

void
on_generate_edit_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
        char *ptr;
        int l;
   
        if (texture_file_name && (l=strlen(texture_file_name))>2 &&
	    !strcmp(texture_file_name+l-2, ".c")) 
          ptr = texture_file_name;
        else
          ptr = "FILENAME";
        if (!texture_editor1)
          texture_editor1 = GTK_editor(ptr, (void *)load_extern_texture);
        else {
          gtk_widget_show(texture_editor1);
	  GTK_editor_set_file(ptr);
	}   
}

void
on_generate_show_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	if(!GFX_Texture.Data)
        	on_generate_random_button_clicked(NULL, NULL);
	on_texture_show_button_clicked(NULL, NULL);
}

void
on_generate_close_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (generate_window1);
}

gboolean
on_resize_window1_delete_event         (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  GtkWidget *editablemax;	      
  editablemax = lookup_widget (GTK_WIDGET (resize_window1), "maxwidth_spinbutton");
  widthmax = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (editablemax));

  gtk_widget_hide (resize_window1);   
  return TRUE;
}

void
on_resize_cut_button_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	if(GFX_Texture.Data) {
		int width;
		GtkWidget *editable;
		GtkWidget *editablemax;	   

		texture_file_name = NULL;
		if(texture_open_dialogue1)
			gtk_widget_hide (texture_open_dialogue1);

		/* destroy former image view */
		if(texture_gfx_window1) {
			gtk_widget_destroy(texture_gfx_window1);
			texture_gfx_window1 = NULL;
		}
		if(tex_align_window1) {
			gtk_widget_destroy(tex_align_window1);
			tex_align_window1 = NULL;
		}

		// the following two lines aren't necessary because the texture width can only be decreased and not increased.
		//Renderer_Params.StartX = 0;
		//Renderer_Params.StartY = 0;

		editable = lookup_widget (GTK_WIDGET (resize_window1), "newwidth_spinbutton");
		editablemax = lookup_widget (GTK_WIDGET (resize_window1), "maxwidth_spinbutton");
		width = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (editable));
		widthmax = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (editablemax));
	        if (width > widthmax) width = widthmax;
		printf("new expected width: %i\n", width);
		if(!GFX_Resize( &GFX_Texture , width)) {
			/* texture_gfx_window1 = create_my_gfx_window1 ( &GFX_Texture , "texture_gfx_window1", "texture image"); */
			/* gtk_widget_show (texture_gfx_window1); */
		} else {
			// error reading gfx data
			GtkWidget *dialog;
			if(GFX_Texture.Data) {
				free(GFX_Texture.Data);
				GFX_Texture.Data = NULL;
			}
			dialog = create_my_msg_dialog1 ("GFX IO Error", "\nCould not resize texture image,\nperhaps not enough free memory available.");
			gtk_widget_show(dialog);
		}
        } else {
		GtkWidget *dialog;
		dialog = create_my_msg_dialog1 ("resize Error", "\nNo texture present.\n\nYou must first of all define a texture.\n");
		gtk_widget_show(dialog);
	}
	gtk_widget_destroy (resize_window1);
	resize_window1 = NULL;
}


void
on_base_invert_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	if(GFX_Base.Data) {
	   /*
		if(tex_align_window1) {
			gtk_widget_destroy(tex_align_window1);
			tex_align_window1 = NULL;
		}
	    */
		GFX_Invert( &GFX_Base );

	        if(!base_gfx_window1)
			base_gfx_window1 = create_my_gfx_window1 ( &GFX_Base , "base_gfx_window1", "base image");
		else {
			GtkWidget *widget;
			widget = lookup_widget (GTK_WIDGET (base_gfx_window1), "drawingarea1");
			on_darea_expose(widget, NULL,  &GFX_Base );
		}
		if(tex_align_window1) {
			GtkWidget *tex_align_drawingarea1;
			tex_align_drawingarea1 = lookup_widget (GTK_WIDGET (tex_align_window1), "tex_align_drawingarea1");
			on_darea_expose(tex_align_drawingarea1, NULL, gfx_tex_to_merge[0]);
		   /*
			gtk_widget_show (tex_align_window1);
			gtk_widget_activate (tex_align_window1);
		    */
		}
	   /*	   
		gtk_widget_show (base_gfx_window1);
		gtk_widget_activate (base_gfx_window1);
	    */
	} else {
		GtkWidget *dialog;
		dialog = create_my_msg_dialog1 ("Error", "\nNo base present.\n\nYou must first of all load a base image.\n");
		gtk_widget_show(dialog);
	}
}
