/* ge -- gtk editor
 * Copyright 1999 David Vogler <david@rhlx01.rz.fht-esslingen.de>
 *		  Adrian Reber <adrian@rhlx01.rz.fht-esslingen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "mode.h"
#include "ge.h"

extern char file[1024];
extern gchar *FileName;
extern GtkStyle *style;
extern GdkFont *font;
GdkFont *GPL_font=NULL;
extern GtkWidget * main_window;
gint changed=1;
gint from_beginning=0;

void insert_text(char* string, gint length)
{
  gint position=0;
  gtk_text_freeze(GTK_TEXT(text));
  position=gtk_editable_get_position(GTK_EDITABLE(text));
  gtk_editable_insert_text(GTK_EDITABLE(text),string,length,&position);
  gtk_text_thaw(GTK_TEXT(text));
}

void insert_text_around (gpointer text ,struct change change, char *string ,gint length)
{
  gchar * buffer;
  gchar * another_buffer;
  gint i=0;
  struct change help;
  help=change;
  /*now filling another_buffer with the starting tag*/
  another_buffer=g_malloc(length+9);
  another_buffer[0]='<';
  for (i=1;i<length+1;i++)
    {
      another_buffer[i]=string[i-1];
    }
  another_buffer[length+1]='>';
  another_buffer[length+2]='\0';
  buffer = gtk_editable_get_chars(text,change.start,change.end);
  gtk_editable_delete_selection(text);
  gtk_editable_insert_text(text,another_buffer,length+2,&change.start);
  gtk_editable_insert_text(text,buffer,strlen(buffer),&change.start);
  /*now filling another_buffer with the ending tag*/
  another_buffer[0]='<';
  another_buffer[1]='/';
  for (i=2;i<length+2;i++)
    {
      another_buffer[i]=string[i-2];
    }
  another_buffer[length+2]='>';
  another_buffer[length+3]='\0';
  gtk_editable_insert_text(text,another_buffer,length+3,&change.start);
  GTK_EDITABLE(text)->selection_start_pos=help.start;
  GTK_EDITABLE(text)->selection_end_pos=help.end;
  GTK_EDITABLE(text)->has_selection=1;
  move_cursor(text,length+2,RIGHT);
  g_free(buffer);
  g_free(another_buffer);
}

void copy_to_clipboard(GtkWidget *window)
{
  gtk_editable_copy_clipboard(GTK_EDITABLE(text));
}

void cut_to_clipboard(GtkWidget *window)
{
  gtk_editable_cut_clipboard(GTK_EDITABLE(text));
}

void paste_from_clipboard(GtkWidget *window)
{
  gtk_editable_paste_clipboard(GTK_EDITABLE(text));
}

struct change test_selection (gpointer text)
{
  struct change change;
  gint help=0;

  if (!GTK_EDITABLE(text)->has_selection)
    {
#ifdef DEBUG
      g_print("\nNow no selection is made.\n");
#endif
      help=gtk_editable_get_position(text);
      change.start=change.end=help;
    }
  else
    {
#ifdef DEBUG
      g_print("\nNow a selection is made.\n");
#endif
      change.start=0;
      change.end=0;
      change.start= GTK_EDITABLE(text)->selection_start_pos;
      change.end = GTK_EDITABLE(text)->selection_end_pos;
      if (change.start>change.end)
	{
	  help=change.start;
	  change.start=change.end;
	  change.end=help;
	}
    }
  return change;
}

gint move_cursor (gpointer text, gint length, GeCursor orientation)
{
  gint txt_len=0;
  struct change change;

  change=test_selection(text);
#ifdef DEBUG
  g_print("\nNow moving cursor : %d",length);
#endif
  if (orientation==LEFT)
   {
     if ((change.start-length)<=0)
       {
	 gtk_editable_set_position(text,0);
       }
     else
       {
	 gtk_editable_set_position(text,change.start-length);
       }
#ifdef DEBUG
     g_print ("\nLEFT\n");
     g_print("\nPosition is %d",change.start);
#endif
   }
  if (orientation==RIGHT)
     {
     txt_len=gtk_text_get_length(text);
     if ((change.start+length)>=txt_len)
       {
	 gtk_editable_set_position(text,txt_len);
       }
     else
       {
	 gtk_editable_set_position(text,change.start+length);
       }
#ifdef DEBUG
     g_print ("\nRIGHT\n");
     g_print("\nPosition is %d",change.start);
#endif
   }
  else
    {
      return 42;
    }
  return 0;
}

void file_ok_sel (GtkWidget *widget, GtkFileSelection *fs)
{
  GString * only_the_file;

  only_the_file = g_string_new("<img src=\"");
  only_the_file = g_string_append(only_the_file,g_basename(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs))));
  only_the_file = g_string_append(only_the_file,"\">");

  insert_text(only_the_file->str,only_the_file->len);
  move_cursor(text,only_the_file->len,RIGHT);
  g_string_free(only_the_file,TRUE);
  gtk_widget_destroy(GTK_WIDGET(fs));
}

GdkColor* get_main_tooltips_bgcolor(GdkWindow *window, GdkColormap *colormap)
{
  static GdkColor color;
  static gboolean alloced = FALSE;
  if (!alloced)
    {
      color.red = 0;
      color.green = 0;
      color.blue = 0;
      color.pixel = 2000;
      gdk_color_alloc(colormap, &color);
      alloced = TRUE;
    }
  return &color;
}

void quit_program (GtkWidget *widget, gpointer data)
{
#ifdef LIBRARY
        gtk_widget_hide (gtk_widget_get_toplevel(widget));
#else
  	if (style)
    	gtk_style_unref (style);
  	if (font)
    	gdk_font_unref (font);
#ifdef DEBUG
  	g_print("\n quit event occured\n");
#endif
  	gtk_main_quit ();
#endif
}

gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
#ifdef DEBUG
  g_print ("delete event occured\n");
#endif
  return (TRUE);
}

void
destroy_message(GtkWidget *dialog)
{
  gtk_grab_remove(dialog);
  gtk_widget_destroy(dialog);
}

void check_key(GtkWidget *widget, GdkEventKey *key)
{
  if ((key->keyval >= 0x20) && (key->keyval <= 0xFF))
    {
#ifdef DEBUG
	g_print("\nI got a %c", key->keyval);
#endif
	if  ((key->keyval == 0xE4))
		{
		 insert_text("&auml;",6);
		 key->length = 0;/* length ist set to 0 so no char appears */
	}}
  else
  #ifdef DEBUG
	g_print("\nI got a %d", key->keyval);
#endif
if (key->keyval==LCTRL)
    {
#ifdef DEBUG
  g_print("\nThats a LCTRL");
#endif
    }
}

void find_entry_changed(GtkWidget *widget)
{
	from_beginning=0;
}

void open_file_ok(GtkWidget *widget, GtkFileSelection *fs)
{
  gchar ge[1025];
  FILE *fd;
  gint length=0;

  gtk_text_freeze(GTK_TEXT(text));
  length=gtk_text_get_length(GTK_TEXT(text));
#ifdef DEBUG
  g_print("\nLength of Text : %d",length);
#endif
  gtk_editable_delete_text(GTK_EDITABLE(text),0,length);
  fd = fopen(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)),"r");
#ifdef DEBUG
  g_print("\nFilename is set to : %s",gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
#endif
  strcpy(file,gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
  if(fd==NULL)
    {
#ifdef DEBUG
      g_print("Can't open file for reading\n");
#endif
      gtk_text_thaw(GTK_TEXT(text));
      return;
    }
  while(fgets(ge, 1024, fd)!=NULL)
    {
      gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL,
		       ge, strlen (ge));
    }
  fclose(fd);
  set_title_name();
  gtk_text_thaw(GTK_TEXT(text));
  gtk_grab_remove(GTK_WIDGET(fs));
  gtk_widget_destroy(GTK_WIDGET(fs));
}

void save_file_ok(GtkWidget *widget, GtkFileSelection *fs)
{
if (does_file_exist(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs))))
{
	if (is_file_a_directory(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs))))
	{
		gtk_file_selection_set_filename (GTK_FILE_SELECTION(fs),strcat(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)),"/"));
      		return;
	}
	if (is_file_readonly(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs))))
	{
	 display_dialog("Error","Insufficient permissions bla\n to write that file");
	return;
	}
	strcpy(file,gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
	gtk_widget_destroy(GTK_WIDGET(fs));
	save_file();

}
else
{
 if (is_file_readonly(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs))))
	{
	 display_dialog("Error","Insufficient permissions bla\n to write that file");
	return;
	}
	strcpy(file,gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
	gtk_widget_destroy(GTK_WIDGET(fs));
	save_file();
}
#ifdef DEBUG
g_print("\ngboolean does_file_exist(gchar *) : %d",does_file_exist(file));
g_print("\ngboolean is_file_readonly(gchar *) : %d",does_file_exist(file));
g_print("\ngboolean is_file_a_directory(gchar *) : %d",does_file_exist(file));
#endif
}

void new_file (void)
{
  gchar * title;
  gint length=0;

  gtk_text_freeze(GTK_TEXT(text));
  length=gtk_text_get_length(GTK_TEXT(text));
  gtk_editable_delete_text(GTK_EDITABLE(text),0,length);
  gtk_text_thaw(GTK_TEXT(text));

  strcpy(file,"FILENAME");
  title=g_malloc(strlen(NAME)+strlen("Untitled")+3);
  strcpy(title,NAME);
  strncat(title," - ",3);
  strncat(title,"Untitled",strlen("Untitled"));
  gtk_window_set_title (GTK_WINDOW (main_window), title);
  g_free(title);
}

void close_message_dialog (GtkWidget *widget, gpointer data)
{
  GtkWidget *dialog = (GtkWidget *) data;
  gtk_widget_destroy (dialog);
}

void ungrab_message(GtkWidget *widget, gpointer data)
{
  gtk_grab_remove (widget);
}

void display_dialog (gchar *title, gchar *message)
{
  GtkWidget *label;
  GtkWidget *button;
  GtkWidget *dialog;

  dialog = gtk_dialog_new ();
  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
		      GTK_SIGNAL_FUNC (ungrab_message),
		      NULL);
  gtk_window_set_title (GTK_WINDOW (dialog), title);
  gtk_container_border_width (GTK_CONTAINER (dialog), 0);

  button = gtk_button_new_with_label ("Make it so!");

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (close_message_dialog),
		      dialog);

  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
		      button, FALSE, FALSE, 0);
  gtk_widget_grab_default (button);
  gtk_widget_show (button);

  label = gtk_label_new (message);

  gtk_misc_set_padding (GTK_MISC (label), 10, 10);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
		      label, TRUE, TRUE, 0);

  gtk_widget_show (label);

  gtk_widget_show (dialog);

  gtk_grab_add (dialog);
}


void find_function (GtkWidget * widget, gpointer data)
{
  gint n;
  gchar * the_text;

#ifdef DEBUG
g_print("\nCase sensitive? %d",gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(find_options.case_sensitive)));
g_print("\nFrom beginning? %d",gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(find_options.from_beginning)));
#endif
  the_text = gtk_editable_get_chars (
				     GTK_EDITABLE (text),
				     (gint) 0,
				     (gint) gtk_text_get_length (GTK_TEXT (text)));
  if (to_be_searched) {
    g_free (to_be_searched);
  }
  to_be_searched = gtk_editable_get_chars (
					   GTK_EDITABLE (data), 0, -1);
  if(!strlen(to_be_searched))
	return;
  n = GTK_EDITABLE (text)->selection_end_pos;
  /* --- Find the string --- */
  n = look_for_string (the_text, n);
  if (n >= 0)
    {
      gtk_text_set_point (GTK_TEXT (text), n);
      gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL, " ", 1);
      gtk_text_backward_delete (GTK_TEXT (text), 1);

      gtk_editable_select_region (GTK_EDITABLE (text),
				  n, n + strlen (to_be_searched));
      widget = NULL;
    }
  else
    {
      display_dialog ("Find...", "Not found.  Reached the end of the file.");
    }
  g_free(the_text);

}

gint look_for_string (gchar *the_text, gint n)
{
  gint text_length;
  gint length;
  gint pos;
  gchar * to_be_searched_2, * the_text_2;

  text_length = (gint) strlen (the_text);
  length = (gint) strlen (to_be_searched);
  to_be_searched_2=g_malloc(length);
  the_text_2=g_malloc(length+1);
  strcpy(to_be_searched_2,to_be_searched);
#ifdef DEBUG
	g_print("\nfrom_beginning : %d", from_beginning);
#endif
  if (!from_beginning && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(find_options.from_beginning)))
	{
		 n=0;
		from_beginning=1;
	}
  for (pos = n; pos < text_length-length; pos++)
{
   strncpy(the_text_2,&the_text[pos],length);
   the_text_2[length]='\0';

  if(!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(find_options.case_sensitive)))
  {  g_strdown(the_text_2);
   g_strdown(to_be_searched_2);
#ifdef DEBUG
	g_print("\nthe_text_2 : %s",the_text_2);
#endif
   if (!strcmp (the_text_2, to_be_searched_2))
    {
      g_free(the_text_2);
      g_free(to_be_searched_2);
      return (pos);
    }
  }
else
  {
#ifdef DEBUG
	g_print("\nthe_text_2 : %s",the_text_2);
#endif
   if (!strcmp(the_text_2, to_be_searched_2))
    {
      g_free(the_text_2);
      g_free(to_be_searched_2);
      return (pos);
    }
  }

}
  g_free(the_text_2);
  g_free(to_be_searched_2);
  return (-1);
}

GtkWidget * entry_in_frame (gchar * title, struct frame_entry * frame_entry)
{
  GtkWidget * entry_box;
  frame_entry->frame = gtk_frame_new(title);
  entry_box = gtk_hbox_new (TRUE, TRUE);
  gtk_container_add (GTK_CONTAINER (frame_entry->frame), entry_box);
  frame_entry->entry = gtk_entry_new ();
  gtk_widget_show (frame_entry->entry);
  gtk_box_pack_start(GTK_BOX(entry_box),frame_entry->entry,FALSE,FALSE,30);
  gtk_widget_show(entry_box);
  gtk_widget_show(frame_entry->frame);
  return entry_box;
}

void set_title_name()
{
	gchar *title;
#ifdef DEBUG
	g_print("\nfilename : %s" , file);
#endif
 	title=g_malloc(strlen(NAME)+strlen(g_basename(file))+3);
	strcpy(title,NAME);
	strncat(title," - ",3);
	strncat(title,g_basename(file),strlen(g_basename(file)));
	gtk_window_set_title (GTK_WINDOW (main_window), title);
	g_free(title);
}

gboolean does_file_exist(gchar *filename)
{
	struct stat buf;
	if (stat(filename, &buf) == -1)
		return FALSE;
	return TRUE;
}

gboolean is_file_readonly(gchar *filename)
{
	if (strcmp(filename, "FILENAME") == 0 || access(filename, F_OK))
		return FALSE;
	return (access(filename, W_OK) != 0);
}

gboolean is_file_a_directory (gchar *filename)
{
struct stat buf;
stat(filename,&buf);
if (buf.st_mode & S_IFREG) return FALSE;
return TRUE;
}

